#include "variables_list.h"
#include <QInputDialog>


VariableList::VariableList(QWidget * parent):BaseWidget(parent)
{
	setWindowIcon( QIcon( QString( ICON_PATH )+"/taskbar.png" ) );
	setWindowTitle("Variables' List");
	
	QVBoxLayout *vbox=new QVBoxLayout;
	setLayout(vbox);
	
	QHBoxLayout *hbox=new QHBoxLayout;
	vbox->addLayout(hbox);
	
	reload_button=new QPushButton(this);
	reload_button->setIcon( QIcon( QString( ICON_PATH )+"/reload.png" ) );
	reload_button->setToolTip("Reload variables' list");
	hbox->addWidget(reload_button);
	reload_button->show();
	connect(reload_button,SIGNAL(clicked ()), this, SLOT(send_whos_command_to_octave()));
	
	//text=new QTextEdit(this);
	//vbox->addWidget(text);
	//text->show();
	
	tree=new QTreeWidget(this);
	QStringList labels;
	labels << "Name" << "Size" << "Bytes" << "Class" << "Prot" ;
	tree->setHeaderLabels(labels);
	connect(tree,SIGNAL(itemDoubleClicked ( QTreeWidgetItem * , int  )), this, SLOT(item_click(QTreeWidgetItem *,int)) );
	
	vbox->addWidget(tree);
	tree->show();
}

void VariableList::setOctaveConnection(OctaveConnection *octave_connection)
{
	this->octave_connection=octave_connection;
	connect(octave_connection,SIGNAL(line_ready(QString)),this,SLOT(line_ready(QString)));
}

void VariableList::line_ready(QString line)
{
	if( line.startsWith("~~whos:") )
	{
		QString s(line);
		s.remove(0,7);
		if( s.trimmed().isEmpty() ) return;
		
		if( s.startsWith("*** dynamically") )
		{
			//text->append("<b>"+s+"</b>");
			s.remove(0,4);
			
			last_root_item=dynamically_item;
			last_root_item->setText(0,s.trimmed());
			last_root_item->setToolTip(0,s.trimmed());
			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/connect_established.png"  ) );
			QFont font(tree->font());
			font.setBold(true);
			last_root_item->setFont(0,font);
		}
		else if( s.startsWith("*** local") )
		{
			//text->append("<b>"+s+"</b>");
			s.remove(0,4);
			
			last_root_item=local_item;
			last_root_item->setText(0,s.trimmed());
			last_root_item->setToolTip(0,s.trimmed());
			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/personal.png"  ) );
			QFont font(tree->font());
			font.setBold(true);
			last_root_item->setFont(0,font);
		}
		else if( s.startsWith("*** currently") )
		{
			//text->append("<b>"+s+"</b>");
			s.remove(0,4);
			
			last_root_item=currently_item;
			last_root_item->setText(0,s.trimmed());
			last_root_item->setToolTip(0,s.trimmed());
			last_root_item->setIcon(0,QIcon( QString( ICON_PATH )+"/run.png"  ) );
			QFont font(tree->font());
			font.setBold(true);
			last_root_item->setFont(0,font);
		}
		else if( s.startsWith("   ") )
		{
			//text->append("<pre>"+s+"</pre>");
			QTreeWidgetItem *item=new QTreeWidgetItem(last_root_item);
			QStringList list = s.trimmed().split(QRegExp("\\s+"));
			int i=0;
			for(;i<list.size() && i<4;i++)
			{
				item->setText(i,list.at(i+1) );
				item->setToolTip(i,list.at(i+1) );
			}
			s=list.at(i++);
			for(;i<list.size();i++) s+=list.at(i);
			item->setText(3,s);
			item->setToolTip(3,s);
			item->setText(4,list.at(0) );
			item->setToolTip(4,list.at(0) );
			//tree->resizeColumnToContents(1);
			//tree->resizeColumnToContents(2);
			for(i=1;i<5;i++) tree->resizeColumnToContents(i);
		}
		else if( s.startsWith("  ") )
		{
			//text->append("<pre>"+s+"</pre>");
		}
		else
		{
			//text->append(s);
			QTreeWidgetItem *item=new QTreeWidgetItem(last_root_item);
			item->setText(0,s.trimmed());
			item->setToolTip(0,s.trimmed());
		}
	}else if (line.startsWith("~~scalar:"))
	{
	        QStringList sl = line.mid(9).split("=");
		bool ok;

		sl.last() = QInputDialog::getText(this, sl.first(), QString(),
						  QLineEdit::Normal, sl.last().trimmed(),
						  &ok);

		if(ok && !sl.last().isEmpty())
		  octave_connection->command_enter(sl.first() + " = " + sl.last());
	}else if (line.startsWith("~~string:"))
	{
	        QString s     = line.mid(9);
	        QString var   = s.section("=", 0, 0);

		s = s.section("=", 1);

		QString value = s.left(s.length() - 1);

		bool ok;

		value = QInputDialog::getText(this, var, QString(),
					      QLineEdit::Normal, value,
					      &ok);

		if(ok)
		  octave_connection->command_enter(var + " = \"" + value + "\"");
	}
}

void VariableList::windowActivated ( QWidget * /*w*/ )
{
	//send_whos_command_to_octave();
}

void VariableList::windowActivated()
{
	//send_whos_command_to_octave();
}

void VariableList::send_whos_command_to_octave()
{
	//text->clear();
	
	tree->clear();
	
	local_item=new QTreeWidgetItem(tree);
	dynamically_item=new QTreeWidgetItem(tree);
	currently_item=new QTreeWidgetItem(tree);
	
	QString command;
	command=
	//"command_line_prompt1=PS1(\"\"); command_line_prompt2=PS2(\"\");\n"
	"if ( 1==1 );"
	//"in = -1;\n"
	//"out = -1;\n"
	//"pid = -1;\n"
	
	"[stdin_pipe, stdin_status] = pipe ();"
	"[stdout_pipe, stdout_status] = pipe ();"
	
	
	"if (stdin_status == 0 && stdout_status == 0);"
	
	
	"	pid = fork ();"
		
		
	"	if (pid == 0);"
			
			
	//"		## In the child.\n"
			
			
	"		fclose (nth (stdin_pipe, 2));"
	"		fclose (nth (stdout_pipe, 1));"
			
			
	"		dup2 (nth (stdin_pipe, 1), stdin);"
	"		fclose (nth (stdin_pipe, 1));"
			
			
	"		dup2 (nth (stdout_pipe, 2), stdout);"
	"		fclose (nth (stdout_pipe, 2));"
	"		clear pid command_line_prompt1 command_line_prompt2 stdin_pipe stdin_status stdout_pipe stdout_status;"	
	"		whos;"
	"		exit(0);"
			
	"	elseif (pid);"
		
	//"		## In the parent.\n"
			
			
	"		fclose (nth (stdin_pipe, 1));"
	"		fclose (nth (stdout_pipe, 2));"
			
	"		in_pipe = nth (stdin_pipe, 2);"
	"		out_pipe = nth (stdout_pipe, 1);"
			
			
	"		fclose(in_pipe);"
			//printf("Leyendo la salida\n");
			
	"		while( feof(out_pipe)!=1 );"
	"		    line_buffer=fgets(out_pipe);"
	"		    if( ischar (line_buffer) );"
	"			fprintf(stderr,\"~~whos:\");"
	"			fputs(stderr,line_buffer);"
	"		    else;"
	"			break;"
	"		    endif;"
	"		end;"
	"		fclose(out_pipe);"
	
	"		clear in_pipe line_buffer out_pipe ;"

			
	"	elseif (pid < 0);"
	"		error (\"octave_whos: fork failed -- unable to create child process\");"
	"	endif;"
	"else;"
	"	error (\"octave_whos: file name must be a string\");"
	"endif; "
	//"PS1(command_line_prompt1); PS2(command_line_prompt2); "
	"endif;"
	;
	octave_connection->command_enter(command,false);
	//octave_connection->command_enter("printf(\"\\n\")",false);
}



void VariableList::item_click(QTreeWidgetItem * item, int /*column*/)
{
	QString name=item->text(0);
	QString clas=item->text(3);
	QString cmd;

	if (clas=="double" || clas=="matrix")
	{
		emit open_table(name);
	}else if (clas == "scalar")
	{
	  cmd = "fprintf(stderr, \"~~scalar:" + name + "=%g\\n\", " + name + ");";
	  octave_connection->command_enter(cmd, false);
	}else if (clas == "string")
	{
	  cmd = "fprintf(stderr, \"~~string:" + name + "=%s\\n\", " + name + ");";
	  octave_connection->command_enter(cmd, false);
	}
}

