/* Copyright (C) 2006 P.L. Lucas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA. 
 */

#ifndef __WINDOWS_LAYOUT_H__
#define __WINDOWS_LAYOUT_H__

#include <QTextEdit>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QListWidget>
#include <QLineEdit>
#include <QLabel>
#include <QCheckBox>
#include <QSplitter>
#include <QWorkspace>
#include <QStackedLayout>
#include "basewidget.h"
#include "config.h"


class AddButton:public QPushButton
{
	Q_OBJECT
	public:
	AddButton( QWidget * parent = 0 );
	/**This is the widget contained by button.*/
	QWidget *widget_contained;
	QStackedLayout *parent_layout;
	
	public slots:
	void clicked_cb();
	void show_widgets();
	
	signals:
	void button_clicked(AddButton *w);
};


/** Builds Windows' layout. User can select several windows and show all of them in only one window.*/

class WindowsLayout:public BaseWidget
{
	Q_OBJECT
	public:
	WindowsLayout( QWidget * parent = 0 );
	void set_window_list(QWidgetList list);
	void set_work_space(QWorkspace *work_space);
	
	public slots:
	void add_widget_cb(AddButton *button);
	void change_split_orientation(int state);
	void show_in_main_window_cb();
	void show_out_main_window_cb();
	void select_icon_cb();
	
	signals:
	void show_widgets();
	
	private:
	QWidgetList list;
	QListWidget *tree;
	QWidget *widget;
	QLineEdit *title;
	QSplitter *split_widget;
	QCheckBox *vertical_split_ok;
	QPushButton *show_in_main_window_button;
	QPushButton *show_out_main_window_button;
	QPushButton *icon_button;
	QWorkspace *work_space;

};




#endif
